# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
import numpy as np
import time
import sys
import argparse
from pai4sk.metrics import accuracy_score
import pydotplus

defaultPath = "."
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
args = CLI.parse_args()

t0 = time.time()
X_train = np.load(args.data_path + "/data/criteo.kaggle.raw.encoded.X_train.npy")
X_test  = np.load(args.data_path + "/data/criteo.kaggle.raw.encoded.X_test.npy")
y_train = np.load(args.data_path + "/data/criteo.kaggle.raw.encoded.y_train.npy")
y_test  = np.load(args.data_path + "/data/criteo.kaggle.raw.encoded.y_test.npy")
print("Data load time (s): {0:.2f}".format(time.time()-t0))

max_depth = 10
n_ex = X_train.shape[0]
 
# Import snap DecisionTreeClassifier from pai4sk module directly
from pai4sk import DecisionTreeClassifier as SnapTree
dt = SnapTree(random_state=0, verbose=False, max_depth=max_depth)

# Training time
t0 = time.time()
dt.fit(X_train[0:n_ex], y_train[0:n_ex])
print("[snap.depth10] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred_test = dt.predict(X_test)
acc_snap = accuracy_score(y_test, pred_test)
print("[snap.depth10] Accuracy score:   {0:.4f}".format(acc_snap))

if not args.skip_sklearn:
    # Import sklearn's DecisionTreeClassifier from pai4sk module directly
    from pai4sk.tree import DecisionTreeClassifier as SklearnTree
    dt = SklearnTree(random_state=0, max_depth=max_depth)

    # Training time
    t0 = time.time()
    dt.fit(X_train[0:n_ex], y_train[0:n_ex])
    print("[sklearn.depth10] Training time (s):  {0:.2f}".format(time.time()-t0))
    
    #from sklearn import tree
    #dot_data = tree.export_graphviz(dt, out_file=None)
    #graph = pydotplus.graph_from_dot_data(dot_data)  
    #graph.write_pdf("sklearn-criteo-raw-dt.pdf")

    # Inference
    pred_test = dt.predict(X_test)

    # Evaluate accuracy_score on test set
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn.depth10] Accuracy score:   {0:.4f}".format(acc_sklearn))

