# SUSY Example

In this example we will train Decision Tree and Random Forest models on the SUSY dataset from the LibSVM repository, using both `scikit-learn` and `snap-ml-local`. 

## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/SUSY.bz2
bunzip2 SUSY.bz2
cd ../
```

## PreProcess the data

We first perform some data pre-processing.

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/preprocess-susy.py`

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-susy/preprocess-susy.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.


## Training and Evaluating a Decision Tree Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-decision-tree.py`

Command to run example using the pre-processed data in specific directory using 40 threads:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-decision-tree.py --data_path path/to/data --num_threads 40
```

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

The following output should be expected on a 40-core x86 machine (Intel(R) Xeon(R) Gold 6230 CPU @ 2.10GHz) with four V100 GPUs attached:
```
Data load time (s):  0.31
[snap.cpu] Training time (s):  4.57
[snap.cpu] Accuracy score:   0.7889
[snap.gpu] Training time (s):  3.11
[snap.gpu] Accuracy score:   0.7889
[sklearn] Training time (s):  213.00
[sklearn] Accuracy score:   0.7892
```

Times will vary depending on the system architecture.

For the decision tree and random forest models, `snap-ml-local` provides a feature called `histograms`: when enabled, 
the values of each feature are grouped into a user-defined number of bins; during training, the splits are evaluated at the bin values instead
of the individual feature values. This can significantly accelerate the training of the model. To enable `histograms`, the user needs to define two
parameters at model instantiation time: `use_histograms` and `hist_nbins`. An example of how to use these parameters
can be found in `$CONDA_PREFIX/pai4sk/snap-ml-local/local-examples/example-susy/example-susy-decision-tree.py`. The `sklearn` library 
does not offer (as of version 0.22) a decision tree / random forest API with `histograms` functionality. Nevertheless, in this example,
we compare the `snap-ml-local` `histograms`-based decision tree with the `sklearn` decision tree and show the benefits of 
using `histograms` both in terms of training time and generalization performance.

## Training and Evaluating a Random Forest Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-random-forest.py`

Command to run example using the pre-processed data in specific directory using 40 threads and 4 GPUs:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-random-forest.py --data_path path/to/data --num_threads 40 --num_gpus 4
```

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

The following output should be expected on a 40-core x86 machine (Intel(R) Xeon(R) Gold 6230 CPU @ 2.10GHz) with four V100 GPUs attached:
```
Data load time (s):  0.31
[snap.cpu] Training time (s):  39.46
[snap.cpu] Accuracy score:   0.7942
[snap.gpu] Training time (s):  27.86
[snap.gpu] Accuracy score:   0.7945
[sklearn] Training time (s):  361.81
[sklearn] Accuracy score:   0.7944
```

Times will vary depending on the system architecture. 

&copy; Copyright IBM Corporation 2018, 2020
