# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
from __future__ import print_function
import time
import sys
import argparse
from pai4sk.metrics.pairwise import earth_movers_distance
from pai4sk.metrics.pairwise import compute_ground_distances
from pai4sk.metrics.pairwise import word_movers_distance
from pai4sk.simsearch.loaders import load_MNIST_with_embeddings
from pai4sk.simsearch.evaluate import evaluate_topk

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)
CLI.add_argument(
   "--use_gpu",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--top_k",
   type=int,
   default=32
)
CLI.add_argument(
   "--optimization_level",
   type=int,
   default=1
)
CLI.add_argument(
   "--num_cpu_threads",
   type=int
)
CLI.add_argument(
   "--use_emd",
   action = 'store_true',
   default=False
)

args = CLI.parse_args()

#######################################
# CREATE EMBEDDING TABLE (COORDINATES OF PIXELS)
W, X, labels_X, ids_X = load_MNIST_with_embeddings(args.data_path + "/data/MNIST/t10k-images-idx3-ubyte", args.data_path + "/data/MNIST/t10k-labels-idx1-ubyte")

W, Y, labels_Y, ids_Y = load_MNIST_with_embeddings(args.data_path + "/data/MNIST/train-images-idx3-ubyte", args.data_path + "/data/MNIST/train-labels-idx1-ubyte")


#######################################
# Run on the GPU/CPU
if (args.use_gpu==True):
    print("Running the GPU library")
else:
    print("Running the CPU library")
start = time.time()
if (args.use_emd==True):
    D_ground = compute_ground_distances(W, use_gpu=args.use_gpu)
    D = earth_movers_distance(D_ground, X, Y, use_gpu=args.use_gpu, optimization_level=args.optimization_level, num_cpu_threads=args.num_cpu_threads)
else:
    D = word_movers_distance(W, X, Y, use_gpu=args.use_gpu, optimization_level=args.optimization_level, num_cpu_threads=args.num_cpu_threads)
end = time.time()
print("Took " + str(end-start) + " seconds")


# EVALUATE TOP-K(for example TOP-32) SEARCH ACCURACY
# Evaluates search accuracy for MNIST images for the top K samples (K in powers of 2 from 1 to K)

k_vec, prec_vec, topk_indices, topk_values = evaluate_topk(D, args.top_k, labels_X, labels_Y)
print("\nTop-K precision results")
print("K:\t\t" + str(k_vec))
print("Precision:\t" + str(prec_vec))

